If kluge_around_eof is < 0, we go into an msleep() loop instead
[rsync/rsync.git] / io.c
diff --git a/io.c b/io.c
index 7e02d34..2a7d776 100644 (file)
--- a/io.c
+++ b/io.c
@@ -56,6 +56,7 @@ extern struct stats stats;
 
 const char phase_unknown[] = "unknown";
 int select_timeout = SELECT_TIMEOUT;
+int ignore_timeout = 0;
 int batch_fd = -1;
 int batch_gen_fd = -1;
 
@@ -75,9 +76,11 @@ int batch_gen_fd = -1;
 const char *io_write_phase = phase_unknown;
 const char *io_read_phase = phase_unknown;
 
-/** Ignore EOF errors while reading a module listing if the remote
-    version is 24 or less. */
-int kludge_around_eof = False;
+/* Ignore an EOF error if non-zero.  We exit if the value is 1 (used while
+ * reading a module listing if the remote version is 24 or less) or go into a
+ * sleep loop if the value is 2 (used by the receiver when it is reading a
+ * potential end-of-transfer keep-alive message that may never come). */
+int kluge_around_eof = 0;
 
 int msg_fd_in = -1;
 int msg_fd_out = -1;
@@ -137,7 +140,7 @@ static void check_timeout(void)
 {
        time_t t;
 
-       if (!io_timeout)
+       if (!io_timeout || ignore_timeout)
                return;
 
        if (!last_io) {
@@ -369,12 +372,16 @@ void io_set_filesfrom_fds(int f_in, int f_out)
  * rsync <2.4.6 sending a list of modules on a server, since the list
  * is terminated by closing the socket. So, for the section of the
  * program where that is a problem (start_socket_client),
* kludge_around_eof is True and we just exit.
  */
 static void whine_about_eof(int fd)
 {
-       if (kludge_around_eof && fd == sock_f_in)
-               exit_cleanup(0);
+       if (kluge_around_eof && fd == sock_f_in) {
+               if (kluge_around_eof > 0)
+                       exit_cleanup(0);
+               while (1)
+                       msleep(20);
+       }
 
        rprintf(FERROR, RSYNC_NAME ": connection unexpectedly closed "
                "(%.0f bytes received so far) [%s]\n",
@@ -634,6 +641,21 @@ void io_end_buffering(void)
 }
 
 
+void maybe_send_keepalive(int allowed_lull, int ndx)
+{
+       if (time(NULL) - last_io >= allowed_lull) {
+               if (!iobuf_out || !iobuf_out_cnt) {
+                       if (protocol_version < 29)
+                               return; /* there's nothing we can do */
+                       write_int(sock_f_out, ndx);
+                       write_shortint(sock_f_out, ITEM_IS_NEW);
+               }
+               if (iobuf_out)
+                       io_flush(NORMAL_FLUSH);
+       }
+}
+
+
 /**
  * Continue trying to read len bytes - don't return until len has been
  * read.
@@ -1014,6 +1036,7 @@ static void writefd_unbuffered(int fd,char *buf,size_t len)
                         * to grab any messages they sent before they died. */
                        while (fd == sock_f_out && io_multiplexing_in) {
                                io_timeout = select_timeout = 30;
+                               ignore_timeout = 0;
                                readfd_unbuffered(sock_f_in, io_filesfrom_buf,
                                                  sizeof io_filesfrom_buf);
                        }
@@ -369,12 +372,16 @@ void io_set_filesfrom_fds(int f_in, int f_out)
  * is terminated by closing the socket. So, for the section of the
  * program where that is a problem (start_socket_client),
- * kludge_around_eof is True and we just exit.
+ * kluge_around_eof is True and we just exit.
  */
 static void whine_about_eof(int fd)